-- ##### TerraLAB - www.terralab.ufop.br
-- Computer Science Departament, Federal Univeristy of Ouro Preto
-- TerraME Prey-Predator dynamic model based on differental equations
-- Tiago Garcia de Senna Carneiro - 07/2012
--
-------------
-- Each cell has different environmental constraints. The deathPreadorRate is different in each cell.
-- Therefore, each cell has a different dynamics.
-------------

--#PARAMETERS
timeStep =  1 -- See the effects of 0.25, 0.125, 0.0625, and 0.03125
INTEGRATION_METHOD = integrationHeun --  vary the integration Method: ("integrationHeun"), "integrationRungeKutta", "integrationEuler"

--#SPATIAL MODEL
rand = Random{ seed = 1}
cs = CellularSpace{ xdim = 10 }
forEachCell( cs, function(cell) 
	cell.preys = 100
	cell.predators = 10
	cell.birthPreyRate = 0.2
	cell.predationRate = 0.01 -- death prey rate
	cell.birthPredatorPerPreyRate = 0.01 -- birth predator rate
	cell.deathPredatorRate =  0.1 + 0.1 * rand:number(0,1) -- death prey rate
end )
cs:synchronize()

--#LEGENDS
preysLeg = Legend{
		-- Attribute name:  height
		type = "number",
		grouping = "equalsteps",
		slices = 100,
		precision = 10,
		stdDeviation = "none",
		maximum = 500,
		minimum = 0,
		colorBar = {
			{color = "white", value = 0},
			{color = "green", value = 500}
		}
}


predatorsLeg = Legend{
		-- Attribute name:  soilWater
		type = "number",
		grouping = "equalsteps",
		slices = 100,
		precision = 10,
		stdDeviation = "none",
		maximum = 500,
		minimum = 0,
		colorBar = {
			{color = "white", value = 0},
			{color = "red", value = 500}
		}
}

--#OBSERVERS
Observer{ subject = cs, type = "map", attributes={"preys", "predators"},legends= { preysLeg, predatorsLeg} }


--#RULES
for t = 0, 4000, timeStep do 

	forEachCell( cs, function( cell )
		
		cell.preys, cell.predators = d{
			{ 	
				function( t, q )  
					return q[1]*cell.birthPreyRate - q[1]*q[2]*cell.predationRate 
				end , 
				function( t, q ) 
					return q[2]*q[1]*cell.birthPredatorPerPreyRate - q[2]*cell.deathPredatorRate
				end
			},
			{ cell.past.preys, cell.past.predators }, 
			0, timeStep, 0.03125 
		}
		
	end)

	cs:synchronize()	
	cs:notify()
	
	print("Time: ", t)
end

